#!/usr/bin/perl -w
use 5.010;
use strict;
use warnings;
use utf8;

use Blio;
use Blio::Node;
use DBI;
use FindBin;
use Path::Class;
use DateTime::Format::Strptime;
use Encode;
use Class::Load qw(try_load_class);
use Sys::Hostname qw(hostname);

my $blio = Blio->new_with_options;

$blio->collect;
handle_vinyl();
handle_microblog();
prepare_index();

$blio->stash->{menu_order} =  [ map {$_.'.html' } qw(
microblog perl bicycle reisen talks var mixtapes potd vinyl tags
)];

$blio->write;


if (try_load_class('Net::Twitter::Lite::WithAPIv1_1') && hostname() eq 'bnr') {
    potd_tweet();
}

sub potd_tweet {
    my %tweeted;
    my $index_file = file($blio->output_dir->file('tweeted.index'));
    if (-e $index_file) {
        %tweeted = map { chomp; $_ =>1 } $index_file->slurp;
    }
    my @to_tweet;

    while (my ($url, $node) = each %{$blio->nodes_by_url}) {
        next unless $url =~ /^potd\//;
        next if $tweeted{$url};
        if (my $tweet = $node->stash->{tweet}) {
            push(@to_tweet, $node);
        }
    }

    if (@to_tweet == 1) {
        my $node = $to_tweet[0];
        my $status = $node->stash->{tweet};
        my $length_check = $status;
        $length_check=~s{https?://.* }{'x' x 24}ge; # twitter short urls are 23 characters + 1 trailing space
        if (length($length_check) > 89) {
            $status=substr($status, 0,87).'…';
        }

        my $message = {
            status=>'#potd '.encode_utf8($status).' '.$blio->site_url.'/'.$node->url,
            media=>[ $blio->output_dir->file($node->primary_image->url)->stringify ]
        };

        my $conf = do $blio->output_dir->parent->file('twitter.conf')->stringify;
        my $nt =  Net::Twitter::Lite::WithAPIv1_1->new(
            clientname => 'Blio',
            clientver => 1,
            clienturl => 'http://domm.plix.at',
            useragent => "Blio/1",
            legacy_lists_api => 0,
            ssl => 1,
            apiurl => 'https://api.twitter.com/1.1',
            %$conf
        );

        eval {
            my $rv = $nt->update_with_media($message);
            if ($rv->{id_str}) {
                say "successfully tweeted: ".$message->{status};
            }
        };
        if ($@) {
            say "error while tweeting: ".$@;
        }

        my $fh = $index_file->opena;
        say $fh $node->url;
        close $fh;
    }
    elsif (@to_tweet > 1) {
        say "more than one tweet, stay silent to be sure";
        say join(', ',map {$_->url} @to_tweet);
    }

}

sub prepare_index {
    my $index = $blio->nodes_by_url->{'index.html'};
    my @featured;
    my $by_date = $blio->nodes_by_date;
    my (@vinyl, @microblog, @nodes, @images, $potd);
    my $full=0;
    foreach my $feature (@$by_date) {
        last if @microblog >= 2 && @vinyl >= 6 && @nodes >= 5 && @images >=5;
        next unless $feature->parent;
        my $url = $feature->url;
        if ($url =~m{^vinyl/}) {
            next if @vinyl >= 6;
            push(@vinyl,$feature);
        }
        elsif ($url =~m{^microblog/}) {
            next if @microblog >= 2;
            push(@microblog,$feature);
        }
        elsif ($url =~m{^potd}) {
            $potd = $feature unless $potd;
        }
        elsif ($url =~m{^tags/}) {
            next;
        }
        else {
            if (@nodes < 5) {
                push(@nodes,$feature);
            }
            if (@images < 5) {
                if ($feature->has_images) {
                    push(@images,$feature);
                }
            }
        }
    }

    if ($potd) {
        splice(@nodes,1,0,$potd);
    }

    $index->stash->{images}=\@images;
    $index->stash->{nodes}=\@nodes;
    $index->stash->{vinyl}=\@vinyl;
    $index->stash->{microblog}=\@microblog;
    $index->stash->{potd}=$potd;

    my @top_atom;
    my $all_sorted = $blio->nodes_by_date;
    foreach (@$all_sorted) {
        next if $_->id =~m{vinyl/};
        next if $_->id =~m{tags/};
        next if $_->id =~m{microblog/};
        next if $_->id =~m{potd/};
        last if @top_atom >=15;
        push(@top_atom,$_);
    }
    $index->children(\@top_atom);
}

sub handle_vinyl {
    my $vinyl = $blio->nodes_by_url->{'vinyl.html'};

    my $vinyldb = $FindBin::Bin.'/sqlite/vinyl.db';
    my $DBH=DBI->connect("dbi:SQLite:dbname=".$vinyldb,{sqlite_unicode => 1});
    my $sth=$DBH->prepare("select * from vinyl order by buy DESC,id DESC");
    $sth->execute;
    
    my @medium=('unknown','Album','12"','10"','7"','CD','MP3');
    
    while (my $r=$sth->fetchrow_hashref) {
        my $title = $r->{artist}.' - '.$r->{name};
        my $id = $title;
        $id =~s/[^\w\d-]+/_/g;
        my $url = 'vinyl/'.$id.'.html';
        my $buy = $r->{buy};
        $buy .= '0101' if length($buy) == 4;
        $buy .= '01' if length($buy) == 6;

        my $date=DateTime->new(
            year=>substr($buy,0,4),
            month=>substr($buy,4,2),
            day=>substr($buy,6,2),
            time_zone=>'local',
        );
        my @content;
        push(@content,$r->{description}) if $r->{description};
        push(@content,$medium[$r->{medium}].', '.$r->{pub});
        push(@content,$r->{shop});
        my $content = join("\n", map { "<p>$_</p>" } @content);

        my $node = Blio::Node->new(
            base_dir => $blio->source_dir,
            source_file => file($vinyldb),
            id=>'vinyl/'.$id,
            url=>$url,
            title=>decode_utf8($title),
            date=>$date,
            content=>decode_utf8($content),
        );
        $blio->nodes_by_url->{$url}=$node;
        $vinyl->add_child($node);
        $node->parent($vinyl);
    }
}

sub handle_microblog {
    my $mb = $blio->nodes_by_url->{'microblog.html'};
    $mb->template('microblog_index.tt');

    my $db = $FindBin::Bin.'/sqlite/microblog.db';
    my $DBH=DBI->connect("dbi:SQLite:dbname=".$db);
    my $sth=$DBH->prepare("select * from microblog order by date DESC,id DESC");
    $sth->execute;
    
    my $strp = DateTime::Format::Strptime->new(
        pattern     => '%FT%T',
    );

    while (my $r=$sth->fetchrow_hashref) {
        my $date = $strp->parse_datetime($r->{date});
        my $id = 'microblog/'.$date->iso8601;
        my $url = $id.'.html';

        my $node = Blio::Node->new(
            base_dir => $blio->source_dir,
            source_file => file($db),
            id=>$id,
            url=>$url,
            title=>$r->{message},
            date=>$date,
            content=>$r->{message},
            template=>'microblog.tt',
        );
        $blio->nodes_by_url->{$url}=$node;
        $mb->add_child($node);
        $node->parent($mb);
    }
}

